.codehilite {
  background: var(--neutral-fill-active);
}
.bk.card button.bk.card-header {
  height: 55px;
}
p.bk.card-button {
  display: block;
  padding-right: 1em;
}
.bk.card {
  color: var(--neutral-foreground-rest);
  background-color: var(--neutral-fill-rest);
}
.bk.card-header {
  color: var(--neutral-foreground-rest);
  background-color: var(--neutral-fill-active) !important;
}
.bk.pn-loading:before {
  position: absolute;
  height: 100%;
  width: 100%;
  content: '';
  z-index: 1000;
  background-color: rgb(255,255,255,0.50);
  border-color: lightgray;
  background-image: url('https://raw.githubusercontent.com/MarcSkovMadsen/awesome-panel-assets/e6cb56375bb1c436975e09739a231fb31e628a63/spinners/default.svg');
  background-repeat: no-repeat;
  background-position: center;
  background-size: auto 50%;
  border-width: 1px;

}
.bk.pn-loading:hover:before {
  cursor: progress;
}
.card-margin {
  margin: 1em;
}
fast-card {
  margin: 0px;
  padding: 1em;
}
canvas, img {
  border-radius: calc(var(--corner-radius) * 1px);
}
